@@ -73,6 +73,37 @@ class DbalLoggerTest extends \PHPUnit_Framework_TestCase
         ));
     }
 
+    public function testLogNonUtf8Array()
+    {
+        $logger = $this->getMock('Psr\\Log\\LoggerInterface');
+
+        $dbalLogger = $this
+            ->getMockBuilder('Symfony\\Bridge\\Doctrine\\Logger\\DbalLogger')
+            ->setConstructorArgs(array($logger, null))
+            ->setMethods(array('log'))
+            ->getMock()
+        ;
+
+        $dbalLogger
+            ->expects($this->once())
+            ->method('log')
+            ->with('SQL', array(
+                    'utf8' => 'foo',
+                    array(
+                        'nonutf8' => DbalLogger::BINARY_DATA_VALUE,
+                    ),
+                )
+            )
+        ;
+
+        $dbalLogger->startQuery('SQL', array(
+            'utf8' => 'foo',
+            array(
+                'nonutf8' => "\x7F\xFF",
+            ),
+        ));
+    }
+
     public function testLogLongString()
     {
         $logger = $this->getMock('Psr\\Log\\LoggerInterface');
@@ -103,10 +134,6 @@ class DbalLoggerTest extends \PHPUnit_Framework_TestCase
 
     public function testLogUTF8LongString()
     {
-        if (!function_exists('mb_detect_encoding')) {
-            $this->markTestSkipped('Testing log shortening of utf8 charsets requires the mb_detect_encoding() function.');
-        }
-
         $logger = $this->getMock('Psr\\Log\\LoggerInterface');
 
         $dbalLogger = $this
@@ -130,7 +157,7 @@ class DbalLoggerTest extends \PHPUnit_Framework_TestCase
         $dbalLogger
             ->expects($this->once())
             ->method('log')
-            ->with('SQL', array('short' => $shortString, 'long' => mb_substr($longString, 0, DbalLogger::MAX_STRING_LENGTH - 6, mb_detect_encoding($longString)).' [...]'))
+            ->with('SQL', array('short' => $shortString, 'long' => mb_substr($longString, 0, DbalLogger::MAX_STRING_LENGTH - 6, 'UTF-8').' [...]'))
         ;
 
         $dbalLogger->startQuery('SQL', array(
