@@ -11,7 +11,9 @@
 
 namespace Symfony\Bundle\FrameworkBundle\Tests\Console\Descriptor;
 
+use Symfony\Component\Console\Input\ArrayInput;
 use Symfony\Component\Console\Output\BufferedOutput;
+use Symfony\Component\Console\Style\SymfonyStyle;
 use Symfony\Component\DependencyInjection\Alias;
 use Symfony\Component\DependencyInjection\ContainerBuilder;
 use Symfony\Component\DependencyInjection\Definition;
@@ -75,6 +77,9 @@ abstract class AbstractDescriptorTest extends \PHPUnit_Framework_TestCase
         $this->assertDescription($expectedDescription, $definition);
     }
 
+    /**
+     * @group legacy
+     */
     public function provideLegacySynchronizedServiceDefinitionTestData()
     {
         return $this->getDescriptionTestData(ObjectsProvider::getLegacyContainerDefinitions());
@@ -113,6 +118,7 @@ abstract class AbstractDescriptorTest extends \PHPUnit_Framework_TestCase
         $data = $this->getDescriptionTestData(ObjectsProvider::getContainerParameter());
 
         $data[0][] = array('parameter' => 'database_name');
+        $data[1][] = array('parameter' => 'twig.form.resources');
 
         return $data;
     }
@@ -146,6 +152,11 @@ abstract class AbstractDescriptorTest extends \PHPUnit_Framework_TestCase
     {
         $options['raw_output'] = true;
         $output = new BufferedOutput(BufferedOutput::VERBOSITY_NORMAL, true);
+
+        if ('txt' === $this->getFormat()) {
+            $options['output'] = new SymfonyStyle(new ArrayInput(array()), $output);
+        }
+
         $this->getDescriptor()->describe($output, $describedObject, $options);
 
         if ('json' === $this->getFormat()) {
