@@ -19,6 +19,8 @@ use Symfony\Component\DependencyInjection\Exception\InactiveScopeException;
 use Symfony\Component\DependencyInjection\Exception\InvalidArgumentException;
 use Symfony\Component\DependencyInjection\Exception\LogicException;
 use Symfony\Component\DependencyInjection\Exception\RuntimeException;
+use Symfony\Component\DependencyInjection\Exception\ServiceCircularReferenceException;
+use Symfony\Component\DependencyInjection\Exception\ServiceNotFoundException;
 use Symfony\Component\DependencyInjection\Extension\ExtensionInterface;
 use Symfony\Component\Config\Resource\FileResource;
 use Symfony\Component\Config\Resource\ResourceInterface;
@@ -31,8 +33,6 @@ use Symfony\Component\ExpressionLanguage\ExpressionFunctionProviderInterface;
  * ContainerBuilder is a DI container that provides an API to easily describe services.
  *
  * @author Fabien Potencier <fabien@symfony.com>
- *
- * @api
  */
 class ContainerBuilder extends Container implements TaggedContainerInterface
 {
@@ -90,6 +90,11 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      */
     private $expressionLanguageProviders = array();
 
+    /**
+     * @var string[] with tag names used by findTaggedServiceIds
+     */
+    private $usedTags = array();
+
     /**
      * Sets the track resources flag.
      *
@@ -127,8 +132,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * Registers an extension.
      *
      * @param ExtensionInterface $extension An extension instance
-     *
-     * @api
      */
     public function registerExtension(ExtensionInterface $extension)
     {
@@ -147,8 +150,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * @return ExtensionInterface An extension instance
      *
      * @throws LogicException if the extension is not registered
-     *
-     * @api
      */
     public function getExtension($name)
     {
@@ -167,8 +168,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * Returns all registered extensions.
      *
      * @return ExtensionInterface[] An array of ExtensionInterface
-     *
-     * @api
      */
     public function getExtensions()
     {
@@ -181,8 +180,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * @param string $name The name of the extension
      *
      * @return bool If the extension exists
-     *
-     * @api
      */
     public function hasExtension($name)
     {
@@ -193,8 +190,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * Returns an array of resources loaded to build this configuration.
      *
      * @return ResourceInterface[] An array of resources
-     *
-     * @api
      */
     public function getResources()
     {
@@ -207,8 +202,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * @param ResourceInterface $resource A resource instance
      *
      * @return ContainerBuilder The current instance
-     *
-     * @api
      */
     public function addResource(ResourceInterface $resource)
     {
@@ -227,8 +220,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * @param ResourceInterface[] $resources An array of resources
      *
      * @return ContainerBuilder The current instance
-     *
-     * @api
      */
     public function setResources(array $resources)
     {
@@ -247,8 +238,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * @param object $object An object instance
      *
      * @return ContainerBuilder The current instance
-     *
-     * @api
      */
     public function addObjectResource($object)
     {
@@ -273,7 +262,9 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
         }
 
         do {
-            $this->addResource(new FileResource($class->getFileName()));
+            if (is_file($class->getFileName())) {
+                $this->addResource(new FileResource($class->getFileName()));
+            }
         } while ($class = $class->getParentClass());
 
         return $this;
@@ -289,8 +280,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      *
      * @throws BadMethodCallException When this ContainerBuilder is frozen
      * @throws \LogicException        if the container is frozen
-     *
-     * @api
      */
     public function loadFromExtension($extension, array $values = array())
     {
@@ -312,8 +301,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * @param string                $type The type of compiler pass
      *
      * @return ContainerBuilder The current instance
-     *
-     * @api
      */
     public function addCompilerPass(CompilerPassInterface $pass, $type = PassConfig::TYPE_BEFORE_OPTIMIZATION)
     {
@@ -328,8 +315,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * Returns the compiler pass config which can then be modified.
      *
      * @return PassConfig The compiler pass config
-     *
-     * @api
      */
     public function getCompilerPassConfig()
     {
@@ -340,8 +325,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * Returns the compiler.
      *
      * @return Compiler The compiler
-     *
-     * @api
      */
     public function getCompiler()
     {
@@ -357,10 +340,14 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      *
      * @return array An array of scopes
      *
-     * @api
+     * @deprecated since version 2.8, to be removed in 3.0.
      */
-    public function getScopes()
+    public function getScopes($triggerDeprecationError = true)
     {
+        if ($triggerDeprecationError) {
+            @trigger_error('The '.__METHOD__.' method is deprecated since version 2.8 and will be removed in 3.0.', E_USER_DEPRECATED);
+        }
+
         return $this->scopes;
     }
 
@@ -369,23 +356,27 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      *
      * @return array An array of scope children.
      *
-     * @api
+     * @deprecated since version 2.8, to be removed in 3.0.
      */
-    public function getScopeChildren()
+    public function getScopeChildren($triggerDeprecationError = true)
     {
+        if ($triggerDeprecationError) {
+            @trigger_error('The '.__METHOD__.' method is deprecated since version 2.8 and will be removed in 3.0.', E_USER_DEPRECATED);
+        }
+
         return $this->scopeChildren;
     }
 
     /**
      * Sets a service.
      *
+     * Note: The $scope parameter is deprecated since version 2.8 and will be removed in 3.0.
+     *
      * @param string $id      The service identifier
      * @param object $service The service instance
      * @param string $scope   The scope
      *
      * @throws BadMethodCallException When this ContainerBuilder is frozen
-     *
-     * @api
      */
     public function set($id, $service, $scope = self::SCOPE_CONTAINER)
     {
@@ -421,8 +412,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * Removes a service definition.
      *
      * @param string $id The service identifier
-     *
-     * @api
      */
     public function removeDefinition($id)
     {
@@ -435,8 +424,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * @param string $id The service identifier
      *
      * @return bool true if the service is defined, false otherwise
-     *
-     * @api
      */
     public function has($id)
     {
@@ -453,14 +440,12 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      *
      * @return object The associated service
      *
-     * @throws InvalidArgumentException when no definitions are available
-     * @throws InactiveScopeException   when the current scope is not active
-     * @throws LogicException           when a circular dependency is detected
+     * @throws InvalidArgumentException          when no definitions are available
+     * @throws ServiceCircularReferenceException When a circular reference is detected
+     * @throws ServiceNotFoundException          When the service is not defined
      * @throws \Exception
      *
      * @see Reference
-     *
-     * @api
      */
     public function get($id, $invalidBehavior = ContainerInterface::EXCEPTION_ON_INVALID_REFERENCE)
     {
@@ -476,7 +461,7 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
 
         try {
             $definition = $this->getDefinition($id);
-        } catch (InvalidArgumentException $e) {
+        } catch (ServiceNotFoundException $e) {
             if (ContainerInterface::EXCEPTION_ON_INVALID_REFERENCE !== $invalidBehavior) {
                 return;
             }
@@ -495,6 +480,10 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
                 return;
             }
 
+            throw $e;
+        } catch (\Throwable $e) {
+            unset($this->loading[$id]);
+
             throw $e;
         }
 
@@ -524,8 +513,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * @param ContainerBuilder $container The ContainerBuilder instance to merge.
      *
      * @throws BadMethodCallException When this ContainerBuilder is frozen
-     *
-     * @api
      */
     public function merge(ContainerBuilder $container)
     {
@@ -558,8 +545,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * @param string $name The name of the extension
      *
      * @return array An array of configuration
-     *
-     * @api
      */
     public function getExtensionConfig($name)
     {
@@ -598,8 +583,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      *  * Parameter values are resolved;
      *  * The parameter bag is frozen;
      *  * Extension loading is disabled.
-     *
-     * @api
      */
     public function compile()
     {
@@ -640,8 +623,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * Adds the service aliases.
      *
      * @param array $aliases An array of aliases
-     *
-     * @api
      */
     public function addAliases(array $aliases)
     {
@@ -654,8 +635,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * Sets the service aliases.
      *
      * @param array $aliases An array of aliases
-     *
-     * @api
      */
     public function setAliases(array $aliases)
     {
@@ -671,8 +650,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      *
      * @throws InvalidArgumentException if the id is not a string or an Alias
      * @throws InvalidArgumentException if the alias is for itself
-     *
-     * @api
      */
     public function setAlias($alias, $id)
     {
@@ -697,8 +674,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * Removes an alias.
      *
      * @param string $alias The alias to remove
-     *
-     * @api
      */
     public function removeAlias($alias)
     {
@@ -711,8 +686,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * @param string $id The service identifier
      *
      * @return bool true if the alias exists, false otherwise
-     *
-     * @api
      */
     public function hasAlias($id)
     {
@@ -723,8 +696,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * Gets all defined aliases.
      *
      * @return Alias[] An array of aliases
-     *
-     * @api
      */
     public function getAliases()
     {
@@ -739,8 +710,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * @return Alias An Alias instance
      *
      * @throws InvalidArgumentException if the alias does not exist
-     *
-     * @api
      */
     public function getAlias($id)
     {
@@ -763,8 +732,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * @param string $class The service class
      *
      * @return Definition A Definition instance
-     *
-     * @api
      */
     public function register($id, $class = null)
     {
@@ -775,8 +742,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * Adds the service definitions.
      *
      * @param Definition[] $definitions An array of service definitions
-     *
-     * @api
      */
     public function addDefinitions(array $definitions)
     {
@@ -789,8 +754,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * Sets the service definitions.
      *
      * @param Definition[] $definitions An array of service definitions
-     *
-     * @api
      */
     public function setDefinitions(array $definitions)
     {
@@ -802,8 +765,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * Gets all service definitions.
      *
      * @return Definition[] An array of Definition instances
-     *
-     * @api
      */
     public function getDefinitions()
     {
@@ -819,8 +780,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * @return Definition the service definition
      *
      * @throws BadMethodCallException When this ContainerBuilder is frozen
-     *
-     * @api
      */
     public function setDefinition($id, Definition $definition)
     {
@@ -841,8 +800,6 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * @param string $id The service identifier
      *
      * @return bool true if the service definition exists, false otherwise
-     *
-     * @api
      */
     public function hasDefinition($id)
     {
@@ -856,16 +813,14 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      *
      * @return Definition A Definition instance
      *
-     * @throws InvalidArgumentException if the service definition does not exist
-     *
-     * @api
+     * @throws ServiceNotFoundException if the service definition does not exist
      */
     public function getDefinition($id)
     {
         $id = strtolower($id);
 
         if (!array_key_exists($id, $this->definitions)) {
-            throw new InvalidArgumentException(sprintf('The service definition "%s" does not exist.', $id));
+            throw new ServiceNotFoundException($id);
         }
 
         return $this->definitions[$id];
@@ -880,9 +835,7 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      *
      * @return Definition A Definition instance
      *
-     * @throws InvalidArgumentException if the service definition does not exist
-     *
-     * @api
+     * @throws ServiceNotFoundException if the service definition does not exist
      */
     public function findDefinition($id)
     {
@@ -917,6 +870,10 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
             throw new RuntimeException(sprintf('You have requested a synthetic service ("%s"). The DIC does not know how to construct this service.', $id));
         }
 
+        if ($definition->isDeprecated()) {
+            @trigger_error($definition->getDeprecationMessage($id), E_USER_DEPRECATED);
+        }
+
         if ($tryProxy && $definition->isLazy()) {
             $container = $this;
 
@@ -950,6 +907,14 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
             }
 
             $service = call_user_func_array($factory, $arguments);
+
+            if (!$definition->isDeprecated() && is_array($factory) && is_string($factory[0])) {
+                $r = new \ReflectionClass($factory[0]);
+
+                if (0 < strpos($r->getDocComment(), "\n * @deprecated ")) {
+                    @trigger_error(sprintf('The "%s" service relies on the deprecated "%s" factory class. It should either be deprecated or its factory upgraded.', $id, $r->name), E_USER_DEPRECATED);
+                }
+            }
         } elseif (null !== $definition->getFactoryMethod(false)) {
             if (null !== $definition->getFactoryClass(false)) {
                 $factory = $parameterBag->resolveValue($definition->getFactoryClass(false));
@@ -964,6 +929,10 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
             $r = new \ReflectionClass($parameterBag->resolveValue($definition->getClass()));
 
             $service = null === $r->getConstructor() ? $r->newInstance() : $r->newInstanceArgs($arguments);
+
+            if (!$definition->isDeprecated() && 0 < strpos($r->getDocComment(), "\n * @deprecated ")) {
+                @trigger_error(sprintf('The "%s" service relies on the deprecated "%s" class. It should either be deprecated or its implementation upgraded.', $id, $r->name), E_USER_DEPRECATED);
+            }
         }
 
         if ($tryProxy || !$definition->isLazy()) {
@@ -975,7 +944,7 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
             $this->callMethod($service, $call);
         }
 
-        $properties = $this->resolveServices($parameterBag->resolveValue($definition->getProperties()));
+        $properties = $this->resolveServices($parameterBag->unescapeValue($parameterBag->resolveValue($definition->getProperties())));
         foreach ($properties as $name => $value) {
             $service->$name = $value;
         }
@@ -1012,7 +981,9 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
     public function resolveServices($value)
     {
         if (is_array($value)) {
-            $value = array_map(array($this, 'resolveServices'), $value);
+            foreach ($value as $k => $v) {
+                $value[$k] = $this->resolveServices($v);
+            }
         } elseif ($value instanceof Reference) {
             $value = $this->get((string) $value, $value->getInvalidBehavior());
         } elseif ($value instanceof Definition) {
@@ -1041,11 +1012,10 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      * @param string $name The tag name
      *
      * @return array An array of tags with the tagged service as key, holding a list of attribute arrays.
-     *
-     * @api
      */
     public function findTaggedServiceIds($name)
     {
+        $this->usedTags[] = $name;
         $tags = array();
         foreach ($this->getDefinitions() as $id => $definition) {
             if ($definition->hasTag($name)) {
@@ -1071,6 +1041,16 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
         return array_unique($tags);
     }
 
+    /**
+     * Returns all tags not queried by findTaggedServiceIds.
+     *
+     * @return string[] An array of tags
+     */
+    public function findUnusedTags()
+    {
+        return array_values(array_diff($this->findTags(), $this->usedTags));
+    }
+
     public function addExpressionLanguageProvider(ExpressionFunctionProviderInterface $provider)
     {
         $this->expressionLanguageProviders[] = $provider;
@@ -1162,7 +1142,7 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
             }
         }
 
-        call_user_func_array(array($service, $call[0]), $this->resolveServices($this->getParameterBag()->resolveValue($call[1])));
+        call_user_func_array(array($service, $call[0]), $this->resolveServices($this->getParameterBag()->unescapeValue($this->getParameterBag()->resolveValue($call[1]))));
     }
 
     /**
@@ -1176,7 +1156,7 @@ class ContainerBuilder extends Container implements TaggedContainerInterface
      */
     private function shareService(Definition $definition, $service, $id)
     {
-        if (self::SCOPE_PROTOTYPE !== $scope = $definition->getScope()) {
+        if ($definition->isShared() && self::SCOPE_PROTOTYPE !== $scope = $definition->getScope(false)) {
             if (self::SCOPE_CONTAINER !== $scope && !isset($this->scopedServices[$scope])) {
                 throw new InactiveScopeException($id, $scope);
             }
