@@ -12,22 +12,18 @@
 namespace Symfony\Component\Form\Extension\Core\Type;
 
 use Symfony\Component\Form\AbstractType;
-use Symfony\Component\OptionsResolver\OptionsResolverInterface;
+use Symfony\Component\OptionsResolver\OptionsResolver;
 
 class BirthdayType extends AbstractType
 {
     /**
      * {@inheritdoc}
      */
-    public function setDefaultOptions(OptionsResolverInterface $resolver)
+    public function configureOptions(OptionsResolver $resolver)
     {
-        $resolver->setDefaults(array(
-            'years' => range(date('Y') - 120, date('Y')),
-        ));
+        $resolver->setDefault('years', range(date('Y') - 120, date('Y')));
 
-        $resolver->setAllowedTypes(array(
-            'years' => 'array',
-        ));
+        $resolver->setAllowedTypes('years', 'array');
     }
 
     /**
@@ -35,13 +31,21 @@ class BirthdayType extends AbstractType
      */
     public function getParent()
     {
-        return 'date';
+        return __NAMESPACE__.'\DateType';
     }
 
     /**
      * {@inheritdoc}
      */
     public function getName()
+    {
+        return $this->getBlockPrefix();
+    }
+
+    /**
+     * {@inheritdoc}
+     */
+    public function getBlockPrefix()
     {
         return 'birthday';
     }
