@@ -38,15 +38,22 @@ class PreloadedExtension implements FormExtensionInterface
     /**
      * Creates a new preloaded extension.
      *
-     * @param FormTypeInterface[]           $types       The types that the extension should support.
-     * @param array[FormTypeExtensionInterface[]] typeExtensions The type extensions that the extension should support.
-     * @param FormTypeGuesserInterface|null $typeGuesser The guesser that the extension should support.
+     * @param FormTypeInterface[]            $types          The types that the extension should support
+     * @param FormTypeExtensionInterface[][] $typeExtensions The type extensions that the extension should support
+     * @param FormTypeGuesserInterface|null  $typeGuesser    The guesser that the extension should support
      */
     public function __construct(array $types, array $typeExtensions, FormTypeGuesserInterface $typeGuesser = null)
     {
-        $this->types = $types;
         $this->typeExtensions = $typeExtensions;
         $this->typeGuesser = $typeGuesser;
+
+        foreach ($types as $type) {
+            // Up to Symfony 2.8, types were identified by their names
+            $this->types[$type->getName()] = $type;
+
+            // Since Symfony 2.8, types are identified by their FQCN
+            $this->types[get_class($type)] = $type;
+        }
     }
 
     /**
