@@ -43,6 +43,8 @@ use Symfony\Component\Intl\Locale\Locale;
  *
  * @author Igor Wiedler <igor@wiedler.ch>
  * @author Bernhard Schussek <bschussek@gmail.com>
+ *
+ * @internal
  */
 class IntlDateFormatter
 {
@@ -638,8 +640,7 @@ class IntlDateFormatter
         if (self::NONE !== $this->timetype) {
             $patternParts[] = $this->defaultTimeFormats[$this->timetype];
         }
-        $pattern = implode(', ', $patternParts);
 
-        return $pattern;
+        return implode(', ', $patternParts);
     }
 }
