@@ -56,8 +56,6 @@ abstract class AbstractNumberFormatterTest extends \PHPUnit_Framework_TestCase
      */
     public function testFormatCurrencyWithCurrencyStyle($value, $currency, $expected)
     {
-        IntlTestHelper::requireFullIntl($this);
-
         $formatter = $this->getNumberFormatter('en', NumberFormatter::CURRENCY);
         $this->assertEquals($expected, $formatter->formatCurrency($value, $currency));
     }
@@ -66,15 +64,15 @@ abstract class AbstractNumberFormatterTest extends \PHPUnit_Framework_TestCase
     {
         return array(
             array(100, 'ALL', 'ALL100'),
-            array(-100, 'ALL', '(ALL100)'),
+            array(-100, 'ALL', '-ALL100'),
             array(1000.12, 'ALL', 'ALL1,000'),
 
             array(100, 'JPY', '¥100'),
-            array(-100, 'JPY', '(¥100)'),
+            array(-100, 'JPY', '-¥100'),
             array(1000.12, 'JPY', '¥1,000'),
 
             array(100, 'EUR', '€100.00'),
-            array(-100, 'EUR', '(€100.00)'),
+            array(-100, 'EUR', '-€100.00'),
             array(1000.12, 'EUR', '€1,000.12'),
         );
     }
@@ -84,8 +82,6 @@ abstract class AbstractNumberFormatterTest extends \PHPUnit_Framework_TestCase
      */
     public function testFormatCurrencyWithCurrencyStyleCostaRicanColonsRounding($value, $currency, $symbol, $expected)
     {
-        IntlTestHelper::requireFullIntl($this);
-
         $formatter = $this->getNumberFormatter('en', NumberFormatter::CURRENCY);
         $this->assertEquals(sprintf($expected, $symbol), $formatter->formatCurrency($value, $currency));
     }
@@ -94,7 +90,7 @@ abstract class AbstractNumberFormatterTest extends \PHPUnit_Framework_TestCase
     {
         return array(
             array(100, 'CRC', 'CRC', '%s100'),
-            array(-100, 'CRC', 'CRC', '(%s100)'),
+            array(-100, 'CRC', 'CRC', '-%s100'),
             array(1000.12, 'CRC', 'CRC', '%s1,000'),
         );
     }
@@ -104,8 +100,6 @@ abstract class AbstractNumberFormatterTest extends \PHPUnit_Framework_TestCase
      */
     public function testFormatCurrencyWithCurrencyStyleBrazilianRealRounding($value, $currency, $symbol, $expected)
     {
-        IntlTestHelper::requireFullIntl($this);
-
         $formatter = $this->getNumberFormatter('en', NumberFormatter::CURRENCY);
         $this->assertEquals(sprintf($expected, $symbol), $formatter->formatCurrency($value, $currency));
     }
@@ -114,7 +108,7 @@ abstract class AbstractNumberFormatterTest extends \PHPUnit_Framework_TestCase
     {
         return array(
             array(100, 'BRL', 'R', '%s$100.00'),
-            array(-100, 'BRL', 'R', '(%s$100.00)'),
+            array(-100, 'BRL', 'R', '-%s$100.00'),
             array(1000.12, 'BRL', 'R', '%s$1,000.12'),
 
             // Rounding checks
@@ -133,8 +127,6 @@ abstract class AbstractNumberFormatterTest extends \PHPUnit_Framework_TestCase
      */
     public function testFormatCurrencyWithCurrencyStyleSwissRounding($value, $currency, $symbol, $expected)
     {
-        IntlTestHelper::requireFullIntl($this);
-
         $formatter = $this->getNumberFormatter('en', NumberFormatter::CURRENCY);
         $this->assertEquals(sprintf($expected, $symbol), $formatter->formatCurrency($value, $currency));
     }
@@ -143,7 +135,7 @@ abstract class AbstractNumberFormatterTest extends \PHPUnit_Framework_TestCase
     {
         return array(
             array(100, 'CHF', 'CHF', '%s100.00'),
-            array(-100, 'CHF', 'CHF', '(%s100.00)'),
+            array(-100, 'CHF', 'CHF', '-%s100.00'),
             array(1000.12, 'CHF', 'CHF', '%s1,000.12'),
             array('1000.12', 'CHF', 'CHF', '%s1,000.12'),
 
@@ -224,7 +216,7 @@ abstract class AbstractNumberFormatterTest extends \PHPUnit_Framework_TestCase
         return array(
             array($formatter, 1, '¤1.00'),
             array($formatter, 1.1, '¤1.00'),
-            array($formatter, 2147483648, '(¤2,147,483,648.00)', $message),
+            array($formatter, 2147483648, '-¤2,147,483,648.00', $message),
             array($formatter, -2147483649, '¤2,147,483,647.00', $message),
         );
     }
@@ -269,7 +261,7 @@ abstract class AbstractNumberFormatterTest extends \PHPUnit_Framework_TestCase
             array($formatter, 1, '¤1.00'),
             array($formatter, 1.1, '¤1.00'),
             array($formatter, 2147483648, '¤2,147,483,648.00'),
-            array($formatter, -2147483649, '(¤2,147,483,649.00)'),
+            array($formatter, -2147483649, '-¤2,147,483,649.00'),
         );
     }
 
@@ -610,10 +602,11 @@ abstract class AbstractNumberFormatterTest extends \PHPUnit_Framework_TestCase
     /**
      * @dataProvider parseProvider
      */
-    public function testParse($value, $expected, $message, $expectedPosition)
+    public function testParse($value, $expected, $message, $expectedPosition, $groupingUsed = true)
     {
         $position = 0;
         $formatter = $this->getNumberFormatter('en', NumberFormatter::DECIMAL);
+        $formatter->setAttribute(NumberFormatter::GROUPING_USED, $groupingUsed);
         $parsedValue = $formatter->parse($value, NumberFormatter::TYPE_DOUBLE, $position);
         $this->assertSame($expected, $parsedValue, $message);
         $this->assertSame($expectedPosition, $position, $message);
@@ -639,6 +632,11 @@ abstract class AbstractNumberFormatterTest extends \PHPUnit_Framework_TestCase
         return array(
             array('prefix1', false, '->parse() does not parse a number with a string prefix.', 0),
             array('1.4suffix', (float) 1.4, '->parse() parses a number with a string suffix.', 3),
+            array('-.4suffix', (float) -0.4, '->parse() parses a negative dot float with suffix.', 3),
+            array('-123,4', false, '->parse() does not parse when invalid grouping used.', 6),
+            array('-1234,567', false, '->parse() does not parse when invalid grouping used.', 5),
+            array('-123,,456', false, '->parse() does not parse when invalid grouping used.', 4),
+            array('-123,,456', -123.0, '->parse() parses when grouping is disabled.', 4, false),
         );
     }
 
@@ -688,7 +686,7 @@ abstract class AbstractNumberFormatterTest extends \PHPUnit_Framework_TestCase
 
         // Bug #59597 was fixed on PHP 5.3.14 and 5.4.4
         // The negative PHP_INT_MAX was being converted to float
-        if ((PHP_VERSION_ID < 50400 && PHP_VERSION_ID >= 50314) || PHP_VERSION_ID >= 50404) {
+        if ((PHP_VERSION_ID < 50400 && PHP_VERSION_ID >= 50314) || PHP_VERSION_ID >= 50404 || (extension_loaded('intl') && method_exists('IntlDateFormatter', 'setTimeZone'))) {
             $this->assertInternalType('int', $parsedValue);
         } else {
             $this->assertInternalType('float', $parsedValue);
