@@ -12,6 +12,7 @@
 namespace Symfony\Component\Serializer\Tests\Encoder;
 
 use Symfony\Component\Serializer\Tests\Fixtures\Dummy;
+use Symfony\Component\Serializer\Tests\Fixtures\NormalizableTraversableDummy;
 use Symfony\Component\Serializer\Tests\Fixtures\ScalarDummy;
 use Symfony\Component\Serializer\Encoder\XmlEncoder;
 use Symfony\Component\Serializer\Serializer;
@@ -74,7 +75,7 @@ class XmlEncoderTest extends \PHPUnit_Framework_TestCase
                 '@Type' => 'test',
             ),
             'föo_bär' => 'a',
-            'Bar' => array(1,2,3),
+            'Bar' => array(1, 2, 3),
             'a' => 'b',
         );
         $expected = '<?xml version="1.0"?>'."\n".
@@ -248,6 +249,21 @@ XML;
         $this->assertEquals($expected, $serializer->serialize($array, 'xml', $options));
     }
 
+    public function testEncodeTraversableWhenNormalizable() {
+        $this->encoder = new XmlEncoder();
+        $serializer = new Serializer(array(new CustomNormalizer()), array('xml' => new XmlEncoder()));
+        $this->encoder->setSerializer($serializer);
+
+        $expected = <<<XML
+<?xml version="1.0"?>
+<response><foo>normalizedFoo</foo><bar>normalizedBar</bar></response>
+
+XML;
+
+        $this->assertEquals($expected, $serializer->serialize(new NormalizableTraversableDummy(), 'xml'));
+
+    }
+
     public function testDecode()
     {
         $source = $this->getXmlSource();
@@ -384,7 +400,7 @@ XML;
                 '@Type' => 'test',
             ),
             'föo_bär' => 'a',
-            'Bar' => array(1,2,3),
+            'Bar' => array(1, 2, 3),
             'a' => 'b',
         );
         $expected = array(
@@ -397,7 +413,7 @@ XML;
                 '@Type' => 'test',
             ),
             'föo_bär' => 'a',
-            'Bar' => array(1,2,3),
+            'Bar' => array(1, 2, 3),
             'a' => 'b',
         );
         $xml = $this->encoder->encode($obj, 'xml');
@@ -471,13 +487,13 @@ XML;
             '@xmlns:media' => 'http://search.yahoo.com/mrss/',
             '@xmlns:gd' => 'http://schemas.google.com/g/2005',
             '@xmlns:yt' => 'http://gdata.youtube.com/schemas/2007',
-            'qux' => "1",
-            'app:foo' => "foo",
-            'yt:bar' => array("a", "b"),
+            'qux' => '1',
+            'app:foo' => 'foo',
+            'yt:bar' => array('a', 'b'),
             'media:baz' => array(
-                'media:key' => "val",
-                'media:key2' => "val",
-                'A B' => "bar",
+                'media:key' => 'val',
+                'media:key2' => 'val',
+                'A B' => 'bar',
                 'item' => array(
                     array(
                         'title' => 'title1',
