@@ -51,15 +51,15 @@ class ReflectionCaster
         $a = static::castFunctionAbstract($c, $a, $stub, $isNested);
 
         if (isset($a[$prefix.'parameters'])) {
-            foreach ($a[$prefix.'parameters'] as &$v) {
+            foreach ($a[$prefix.'parameters']->value as &$v) {
                 $param = $v;
-                $v = array();
+                $v = new EnumStub(array());
                 foreach (static::castParameter($param, array(), $stub, true) as $k => $param) {
                     if ("\0" === $k[0]) {
-                        $v[substr($k, 3)] = $param;
+                        $v->value[substr($k, 3)] = $param;
                     }
                 }
-                unset($v['position'], $v['isVariadic'], $v['byReference'], $v);
+                unset($v->value['position'], $v->value['isVariadic'], $v->value['byReference'], $v);
             }
         }
 
@@ -74,6 +74,59 @@ class ReflectionCaster
         return $a;
     }
 
+    public static function castGenerator(\Generator $c, array $a, Stub $stub, $isNested)
+    {
+        return class_exists('ReflectionGenerator', false) ? self::castReflectionGenerator(new \ReflectionGenerator($c), $a, $stub, $isNested) : $a;
+    }
+
+    public static function castType(\ReflectionType $c, array $a, Stub $stub, $isNested)
+    {
+        $prefix = Caster::PREFIX_VIRTUAL;
+
+        $a += array(
+            $prefix.'type' => $c->__toString(),
+            $prefix.'allowsNull' => $c->allowsNull(),
+            $prefix.'isBuiltin' => $c->isBuiltin(),
+        );
+
+        return $a;
+    }
+
+    public static function castReflectionGenerator(\ReflectionGenerator $c, array $a, Stub $stub, $isNested)
+    {
+        $prefix = Caster::PREFIX_VIRTUAL;
+
+        if ($c->getThis()) {
+            $a[$prefix.'this'] = new CutStub($c->getThis());
+        }
+        $x = $c->getFunction();
+        $frame = array(
+            'class' => isset($x->class) ? $x->class : null,
+            'type' => isset($x->class) ? ($x->isStatic() ? '::' : '->') : null,
+            'function' => $x->name,
+            'file' => $c->getExecutingFile(),
+            'line' => $c->getExecutingLine(),
+        );
+        if ($trace = $c->getTrace(DEBUG_BACKTRACE_IGNORE_ARGS)) {
+            $x = new \ReflectionGenerator($c->getExecutingGenerator());
+            array_unshift($trace, array(
+                'function' => 'yield',
+                'file' => $x->getExecutingFile(),
+                'line' => $x->getExecutingLine() - 1,
+            ));
+            $trace[] = $frame;
+            $a[$prefix.'trace'] = new TraceStub($trace, false, 0, -1, -1);
+        } else {
+            $x = new FrameStub($frame, false, true);
+            $x = ExceptionCaster::castFrameStub($x, array(), $x, true);
+            $a[$prefix.'executing'] = new EnumStub(array(
+                $frame['class'].$frame['type'].$frame['function'].'()' => $x[$prefix.'src'],
+            ));
+        }
+
+        return $a;
+    }
+
     public static function castClass(\ReflectionClass $c, array $a, Stub $stub, $isNested, $filter = 0)
     {
         $prefix = Caster::PREFIX_VIRTUAL;
@@ -114,6 +167,9 @@ class ReflectionCaster
             'this' => 'getClosureThis',
         ));
 
+        if (isset($a[$prefix.'returnType'])) {
+            $a[$prefix.'returnType'] = (string) $a[$prefix.'returnType'];
+        }
         if (isset($a[$prefix.'this'])) {
             $a[$prefix.'this'] = new CutStub($a[$prefix.'this']);
         }
@@ -128,18 +184,25 @@ class ReflectionCaster
             }
             $a[$prefix.'parameters'][$k] = $v;
         }
+        if (isset($a[$prefix.'parameters'])) {
+            $a[$prefix.'parameters'] = new EnumStub($a[$prefix.'parameters']);
+        }
 
         if ($v = $c->getStaticVariables()) {
             foreach ($v as $k => &$v) {
                 $a[$prefix.'use']['$'.$k] = &$v;
             }
             unset($v);
+            $a[$prefix.'use'] = new EnumStub($a[$prefix.'use']);
         }
 
         if (!($filter & Caster::EXCLUDE_VERBOSE) && !$isNested) {
             self::addExtra($a, $c);
         }
 
+        // Added by HHVM
+        unset($a[Caster::PREFIX_DYNAMIC.'static']);
+
         return $a;
     }
 
@@ -154,6 +217,9 @@ class ReflectionCaster
     {
         $prefix = Caster::PREFIX_VIRTUAL;
 
+        // Added by HHVM
+        unset($a['info']);
+
         self::addMap($a, $c, array(
             'position' => 'getPosition',
             'isVariadic' => 'isVariadic',
@@ -161,14 +227,20 @@ class ReflectionCaster
         ));
 
         try {
-            if ($c->isArray()) {
-                $a[$prefix.'typeHint'] = 'array';
-            } elseif (method_exists($c, 'isCallable') && $c->isCallable()) {
-                $a[$prefix.'typeHint'] = 'callable';
-            } elseif ($v = $c->getClass()) {
-                $a[$prefix.'typeHint'] = $v->name;
+            if (method_exists($c, 'hasType')) {
+                if ($c->hasType()) {
+                    $a[$prefix.'typeHint'] = $c->getType()->__toString();
+                }
+            } else {
+                $v = explode(' ', $c->__toString(), 6);
+                if (isset($v[5]) && 0 === strspn($v[4], '.&$')) {
+                    $a[$prefix.'typeHint'] = $v[4];
+                }
             }
         } catch (\ReflectionException $e) {
+            if (preg_match('/^Class ([^ ]++) does not exist$/', $e->getMessage(), $m)) {
+                $a[$prefix.'typeHint'] = $m[1];
+            }
         }
 
         try {
@@ -177,6 +249,9 @@ class ReflectionCaster
                 $a[$prefix.'default'] = new ConstStub($c->getDefaultValueConstantName(), $v);
             }
         } catch (\ReflectionException $e) {
+            if (isset($a[$prefix.'typeHint']) && $c->allowsNull()) {
+                $a[$prefix.'default'] = null;
+            }
         }
 
         return $a;
@@ -220,14 +295,18 @@ class ReflectionCaster
 
     private static function addExtra(&$a, \Reflector $c)
     {
-        $a = &$a[Caster::PREFIX_VIRTUAL.'extra'];
+        $x = isset($a[Caster::PREFIX_VIRTUAL.'extra']) ? $a[Caster::PREFIX_VIRTUAL.'extra']->value : array();
 
         if (method_exists($c, 'getFileName') && $m = $c->getFileName()) {
-            $a['file'] = $m;
-            $a['line'] = $c->getStartLine().' to '.$c->getEndLine();
+            $x['file'] = $m;
+            $x['line'] = $c->getStartLine().' to '.$c->getEndLine();
         }
 
-        self::addMap($a, $c, self::$extraMap, '');
+        self::addMap($x, $c, self::$extraMap, '');
+
+        if ($x) {
+            $a[Caster::PREFIX_VIRTUAL.'extra'] = new EnumStub($x);
+        }
     }
 
     private static function addMap(&$a, \Reflector $c, $map, $prefix = Caster::PREFIX_VIRTUAL)
